// M5StickC Wrist Watch
// Written by Theron Wierenga
// January 2021
// theron@wierengaconsulting.com

#include <M5StickC.h>
#include <WiFi.h>
#include <HTTPClient.h>
#include "time.h"

const char* ssid       = "MySpectrumWiFi20-2G";
const char* password   = "shortspoon272";

const char* ntpServer = "pool.ntp.org";
const long  gmtOffset_sec_east = -3600 * 5;      // Eastern Standard Time
const long  gmtOffset_sec_central = -3600 * 6;   // Central Standard Time
const int   daylightOffset_sec = 3600;

const String endpoint = "http://api.openweathermap.org/data/2.5/weather?id=";
const String endpoint2 = "&units=imperial&APPID=";
const String key =      "22759a049ab498051a6d4c7343bd2d08";

RTC_TimeTypeDef RTC_TimeStruct;
RTC_DateTypeDef RTC_DateStruct;
// Variables for time/date
struct tm timeinfo;
unsigned int iyear, imon, ihour, last_sec = 0;
// Variables for weather
String Desc, Name, Deg, Temp, Humid, Speed, Temp0 = "", Humid0="";
String tags[13] = {"main", "description", "temp", "pressure", "humidity", "temp_min", "temp_max",
                   "speed", "deg", "gust", "sunrise", "sunset", "name"};
int num_flag[13] = {0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0}; 
String payload, weather, wtemp;
int code, myHome = 0;
bool debug = false, debug_min = false, ForC = false;
String name_location[9] = {"Tierra Verde", "Norton Shores", "Chicago", "Sparta", "Tampa", "Muskegon", "Naperville", "East Hampton", "Malverne"};
String num_location[9] = {"4175331", "5004005", "4887398", "5010690", "4174757", "5003132", "4903279", "5115871", "5199600" };
String daysOfTheWeek[7] = {"Sunday    ", "Monday    ", "Tuesday   ", "Wednesday ", "Thursday  ", "Friday    ", "Saturday  "};
HTTPClient http;

void setup() 
{
  // Connect to WiFi
  WiFi_Connect();
  // Get time from Internet
  configTime(gmtOffset_sec_east, daylightOffset_sec, ntpServer);
  getLocalTime(&timeinfo);
  Serial.begin(115200);
  // Start the stick
  M5.begin();
  M5.Lcd.setRotation(3);
  M5.Lcd.fillScreen(BLACK);
  
  RTC_TimeTypeDef TimeStruct;
  ihour = timeinfo.tm_hour;
  if (ihour > 12) ihour = ihour - 12;
  TimeStruct.Hours   = ihour;
  TimeStruct.Minutes = timeinfo.tm_min;
  TimeStruct.Seconds = timeinfo.tm_sec;
  M5.Rtc.SetTime(&TimeStruct);
  iyear = timeinfo.tm_year;
  iyear = iyear % 100;
  iyear = iyear + 2000;
  imon = timeinfo.tm_mon + 1;
  RTC_DateTypeDef DateStruct;
  DateStruct.WeekDay = weekday(iyear, imon - 1, timeinfo.tm_mday);
  DateStruct.Month = imon;
  DateStruct.Date = timeinfo.tm_mday;
  DateStruct.Year = iyear;
  M5.Rtc.SetData(&DateStruct);
}

void loop() 
{
  while (RTC_TimeStruct.Seconds == last_sec)
  {
      M5.Rtc.GetTime(&RTC_TimeStruct);
  }
  last_sec = RTC_TimeStruct.Seconds;
  M5.Rtc.GetData(&RTC_DateStruct);
  if (RTC_TimeStruct.Hours > 12) RTC_TimeStruct.Hours -= 12;
  M5.Lcd.setCursor(20, 15);
  M5.Lcd.setTextSize(2);
  if (((RTC_TimeStruct.Seconds /10) % 2) == 1)
  {
      M5.Lcd.print(daysOfTheWeek[RTC_DateStruct.WeekDay]);
  }
  else
  {
      M5.Lcd.printf("%04d-%02d-%02d\n",RTC_DateStruct.Year, RTC_DateStruct.Month,RTC_DateStruct.Date);
  }
 // M5.Lcd.printf("Week: %d\n",RTC_DateStruct.WeekDay);
  M5.Lcd.setCursor(10, 45);
  M5.Lcd.setTextSize(3);
  M5.Lcd.printf("%02d:%02d:%02d\n",RTC_TimeStruct.Hours, RTC_TimeStruct.Minutes, RTC_TimeStruct.Seconds);
  if ((RTC_TimeStruct.Seconds == 1) || (RTC_TimeStruct.Seconds == 31)) screen_Off();
  M5.update();
  if (M5.BtnB.isPressed())
  {
      M5.Lcd.fillScreen(BLACK);
      WiFi_Connect();
  }
  if (M5.BtnA.isPressed())
  {
      screen_On();
      do_weather();
  }
}

void WiFi_Connect()
{
  long tstart = millis() / 1000;
  long tend = tstart;
  WiFi.begin(ssid, password);
  while ((WiFi.status() != WL_CONNECTED) && ((tend - tstart) < 20)) 
  {
    delay(500);
    tend = millis() / 1000;
  }
   if (WiFi.status() != WL_CONNECTED)
   {
      M5.Lcd.setTextColor(RED, BLACK);
   }
   else
   {
      M5.Lcd.setTextColor(BLUE, BLACK);
   }
}

// ***************************************************************************************************
void do_weather()
{
    // Write weather to screen
    // Temp and Humid
    code = getWeatherFromInternet(num_location[myHome], "Home");
    M5.Lcd.fillScreen(BLACK);
    M5.Lcd.setCursor(0, 5);
    M5.Lcd.setTextSize(2);
    M5.Lcd.print(name_location[myHome]);
    M5.Lcd.setCursor(10, 30);
    M5.Lcd.print("Temp: ");
    M5.Lcd.print(Temp);
    M5.Lcd.setCursor(10, 55);
    M5.Lcd.print("Humid: ");
    M5.Lcd.print(Humid);
    delay(4000);
    M5.Lcd.fillScreen(BLACK);
}

//*********************************************************************************************************************
int getWeatherFromInternet(String location, String isitHome)
{
  //  const String endpoint = "http://api.openweathermap.org/data/2.5/weather?id=" + location + "&units=imperial&APPID=";
  //  const String key =      "22759a049ab498051a6d4c7343bd2d08";

    http.begin(endpoint + location + endpoint2 + key); //Specify the URL
    int httpCode = http.GET();  //Make the request

    if (httpCode > 0)
    {
        // Check for the returning code
        payload = http.getString();
        if (debug) 
        {
            Serial.println(payload);
            Serial.print(payload.length());
            Serial.println(" bytes");
        }
        weather = parse_it(payload, isitHome);
     }
     else 
     {
          if (debug) Serial.println("Error on HTTP request");
     }
     
     http.end(); //Free the resources
     return httpCode;
}

//******************************************************************************************************
String parse_it(String tag, String isitHome)
{
    int tags_i = 0, index = 0, val1, val2, last_val;
    float ftemp;
    String tempx = "", last_temp = "";
    String ret_string = isitHome;
    ret_string += ",";
//    String tags[13] = {"main", "description", "temp", "pressure", "humidity", "temp_min", "temp_max",
//                   "speed", "deg", "gust", "sunrise", "sunset", "name"};

    while (tags_i < 13)
    {
        val1 = tag.indexOf(tags[tags_i], 0);
        if (val1 < 0)
        {
            tags_i += 1;
            continue;
        }
        tempx = tag.substring(val1, val1 + tags[tags_i].length());
        if (debug) Serial.println(tags[tags_i]);
        if (tempx == tags[tags_i])
        {
            if (num_flag[tags_i] == 1)    // It's a numeric value
            {
                val1 = val1 + tags[tags_i].length() + 2;
                index = val1;
                val2 = tag.indexOf("\"", index) - 1;
                tempx = tag.substring(val1, val2);
                if (tempx.endsWith("}"))
                {
                  tempx.remove(tempx.length() - 1, 1);
                }     
                if (debug) Serial.print("            ");
                if (debug) Serial.println(tempx);
                if ((tags[tags_i] == "temp") || (tags[tags_i] == "humidity") || (tags[tags_i] == "speed")|| (tags[tags_i] == "deg"))
                {
                    ret_string += ",";
                    ret_string += tempx;
//                    if ((tags[tags_i] == "temp") && (tempx.length() == 5)) 
//                    {
//                        Temp = tempx.substring(0, tempx.length() - 1);    //Is this my problem?
//                    }
                    if (tags[tags_i] == "temp")
                    {
                      Temp = tempx;
                      if (ForC)
                      {
                        ftemp = Temp.toFloat();
                        ftemp = (ftemp - (float)32.0)/(float)1.8;
                        Serial.println(ftemp);
                        Temp = String(ftemp);
                        Serial.println(Temp);
                      }
                    }
                    
                    if (tags[tags_i] == "humidity") Humid = tempx;
                    if (tags[tags_i] == "speed") Speed = tempx;
                    if (tags[tags_i] == "deg") 
                    {
                      // Deg = tempx;
                      float wf = wtemp.toFloat();
                      if (wf < 22.5) Deg = "N";
                      if ((wf >= 22.5) && (wf < 67.5)) Deg = "NE";
                      if ((wf >= 67.5) && (wf < 112.5)) Deg = "E";
                      if ((wf >= 112.5) && (wf < 157.5)) Deg = "SE";
                      if ((wf >= 157.5) && (wf < 202.5)) Deg = "S";
                      if ((wf >= 202.5) && (wf < 247.5)) Deg = "SW";
                      if ((wf >= 247.5) && (wf < 292.5)) Deg = "W";
                      if ((wf >= 292.5) && (wf < 337.5)) Deg = "NW";
                      if (wf >= 337.5) Deg = "N";
                    }
                }
            }
            else                          // It's a String
            {
                index = val1 + tags[tags_i].length() + 1;
                val1 = tag.indexOf("\"", index) + 1;
                index = val1 + 2;
                val2 = tag.indexOf("\"", index);
                tempx = tag.substring(val1, val2);
                if (debug) Serial.print("            ");
                if (debug) Serial.println(tempx);
                if (tags[tags_i] == "description")
                {
                    ret_string += tempx;
                    Desc = tempx;
                    Desc.setCharAt(0, Desc.charAt(0) - 0x20);
                }
                if (tags[tags_i] == "name")
                {
                    ret_string += ",";
                    ret_string += tempx;
                    Name = tempx;
                    if (debug) Serial.println(Name);
                }
            }
            tags_i += 1;
        }
    }
    if (debug) 
    {
      Serial.println(ret_string); 
      Serial.println(Speed);
      Serial.println(Deg);
    }
    return ret_string;
}

// **********************************************************************************
// Calculates weekday
int weekday(int year, int month, int day)
/* Calculate day of week in proleptic Gregorian calendar. Sunday == 0. */
{
  month += 1;
  int adjustment, mm, yy;
  if (year<2000) year+=2000;
  adjustment = (14 - month) / 12;
  mm = month + 12 * adjustment - 2;
  yy = year - adjustment;
  return (day + (13 * mm - 1) / 5 + yy + yy / 4 - yy / 100 + yy / 400) % 7;
}

void screen_Off()
{
    Wire1.beginTransmission(0x34);
    Wire1.write(0x12);
    Wire1.write(0b01001011);  // LDO2, aka OLED_VDD, off
    Wire1.endTransmission();
}

void screen_On()
{
    Wire1.beginTransmission(0x34);
    Wire1.write(0x12);
    Wire1.write(0x4d); // Enable LDO2, aka OLED_VDD
    Wire1.endTransmission();
}
